<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
          "http://www.w3.org/TR/html4/loose.dtd">
<html>
<head>
<title>{blog_title} - {page_title}</title>
<meta name="author" content="{blog_author}">
<meta http-equiv="Content-Type" content="text/html; charset={blog_charset}">
</head>
<body>
<h1>{blog_title}</h1>
<h2>{blog_description}</h2>
<h1>{page_heading}</h1>
{page_entries}
<p><a href="mailto:{blog_email}">Email the author</a></p>
</body>
</html>
